(*************************************************************************
*     Asignatura:     Estructuras de Datos y de la Informacin
*     Proyecto:       FORMATEADOR DE TEXTOS ASCII
*
*     Mdulo:  TAD TipoFichero
*     Fichero:     (x)TAD     ( )Programa    ( )Otro Tipo
*     Autor(es):   Csar Snchez Pmpanas
*     Fecha:       9 / Junio / 1999
*
*     Descripcion:
*        Contiene las operaciones necesarias para manejar el TipoFichero
*        tales como insertar linea, crear fichero, borrar fichero, etc.
************************************************************************)
UNIT Fichero;

INTERFACE
USES

    TipLinea,Crt;

TYPE

    TipoPuntero= ^TipoNodo;
    TipoNodo=   RECORD
               linea:TipoLinea;
               siguiente:TipoPuntero;
               anterior:TipoPuntero;
               END;  {Record}


   TipoFichero = OBJECT

   PROCEDURE CrearFicheroVacio;
   (* POST: Crea un Fichero Vacio*)

   PROCEDURE InsertarLinea;
   (* POST: Inserta un linea a la Estructura de Datos del TAD TipoFichero*)

   PROCEDURE Siguiente;
   (* POST: Recorre la Estructura hacia adelante*)

   PROCEDURE Retroceder;
   (* POST: Recorre la Estructura hacia detras*)

   PROCEDURE Borrar;
   (* POST: Borra la estructura de datos que corresponde al TipoFichero*)

   PROCEDURE InsertarCaracter(car:char);
   (* POST: Llama al procedimiento del Tad TipoLinea para insertar un
    caracter en la linea actual*)

   PROCEDURE InsertarFormato((* in *) formato:TipoFormato;(*in*) aux:byte);
   (* POST: Inserta el Formato en el TAD TipoLinea. La variable aux se utiliza
   para el caso especial del numero de caracteres por linea *)

   FUNCTION ActualCaracter:char;
   (* POST: Llama a la funcion del TAD TipoLinea para que devuelva el caracter
   actual de la linea *)

   FUNCTION ActualPosicion: byte;
   (* POST: Llama a la Funcion del TAD TipoLinea para devolver la posicion
   actual de la linea *)

   FUNCTION MaximoLinea:byte;
   (* POST: Llama a la funcion del TAD TipoLinea para que devuelva el tamao
      maximo de la linea *)

   PROCEDURE MoverActual(int :integer);
   (* POST: Mueve el puntero actual "int" veces a lo largo de la lista *)

   PROCEDURE VolcarFichero(var Fich:text);
   (* POST: Escribimos toda la lista enlazada en un fichero de texto *)

   PROCEDURE BorrarCaracter(tecla:char);
   (* POST: Borra un caracter de la linea actual *)

   FUNCTION NumeroLineas: integer;
   (* POST: Devuelve el numero de lineas que tiene el fichero *)

   PROCEDURE  VisualizarFichero(line:integer);
   (* POST: Visualiza por pantalla el fichero *)

   PROCEDURE Derecha;
   (* POST : Mueve una posicion a la derecha *)

   PROCEDURE Izquierda;
   (* POST : Mueve una posicion a la izquierda *)

   PROCEDURE DefinirFormato (formato : TipoFormato; num:Byte);
   (* POST : Define el formato de una linea *)

   FUNCTION LongitudActual: byte;
   (* POST : Devuelve la longitud actual de la linea *)

   FUNCTION LineaActual: byte;
{   PROCEDURE VisualizarActual;}

   PROCEDURE InsertarCaracterColocado(car:char);

   PROCEDURE ColocarLinea;

   FUNCTION DevolverEspacios: byte;

   PROCEDURE VolcarFicheroResultado(var Fich:text);

   PROCEDURE DefinirFormatoDefecto ((*in*) alin, esp, long: byte);
   PRIVATE
          actual,inicio,final:TipoPuntero;
          cont:Integer;
          espacios: byte;
          espdef, longdef, alindef: byte;
   END;


   IMPLEMENTATION

   PROCEDURE TipoFichero.CrearFicheroVacio;
   BEGIN

           inicio:= NIL;
           final:= NIL;
           actual:= NIL;
           cont:=0;
           espacios:=1;
           longdef := 72;
           espdef := 1;
           alindef := 4;
   END;

   PROCEDURE TipoFichero.InsertarLinea;
   VAR
      nodo,ante: TipoPuntero;
      linea:TipoLinea;
      alin, max, bucle: byte;
      neg : TipoFormato;

   BEGIN

        IF (inicio = NIL) THEN

        BEGIN
               new(nodo);
               linea.CrearLineaVacia(alindef, longdef);
               nodo^.siguiente:=NIL;
               nodo^.linea:= linea;
               nodo^.anterior:= nil;
               actual:=nodo;
               inicio:=nodo;
               cont:= 1;
               final:=nodo;
        END { IF}
        ELSE
            BEGIN

                 alin:=final^.linea.DevolverColocacion;
                 max:= final^.linea.MaximoLinea;
                 IF final^.linea.DevolverNegrita THEN
                    neg:=1
                 ELSE
                     neg:=-1;

                 FOR bucle:=1 TO espacios DO
                 BEGIN
                      new(nodo);
                      linea.CrearLineaVacia (alindef, longdef);
                      final^.siguiente:= nodo;
                      nodo^.siguiente:= NIL;
                      nodo^.linea:= linea;
                      nodo^.anterior:=final;
                      final:=nodo;
                      cont:=cont + 1;
                      actual^.linea.Definirformato(neg,0);
                 END;
                 final^.linea.DefinirFormato (7,max);
                 final^.linea.DefinirFormato (alin,0);
                 final^.linea.DefinirFormato (neg,0);
            END; { ELSE }


   END; { InsertarLinea}


PROCEDURE TipoFichero.Siguiente;
BEGIN
        IF actual^.siguiente <> NIL THEN
           actual:= actual^.siguiente;
END;


PROCEDURE TipoFichero.Retroceder;
BEGIN
        IF actual^.anterior <> NIL THEN
           actual:= actual^.anterior;
END;

PROCEDURE TipoFichero.Borrar;
VAR
nodo1,nodo2:TipoPuntero;

BEGIN
     nodo1:= inicio;

     WHILE nodo1^.siguiente <> NIL DO
     BEGIN
            nodo2:= nodo1;
            nodo1:= nodo1^.siguiente;
            dispose(nodo2);
     END;

            dispose(nodo1);
            actual:=NIL;
            inicio:= NIL;
            final:= NIL;
            cont:=0;

END; { Borrar }

PROCEDURE TipoFichero.InsertarCaracter(car:char);
(* Llama al procedimiento del TipoLinea para insertar un caracter *)
VAR
   frase,aux: string;
{   actual,}linea2:TipoLinea;
    i:integer;
    lisaux: TipoPuntero;
    coloc, alinea: byte;
BEGIN

     lisaux:=actual;
     IF car='' THEN
     BEGIN
          alinea:=lisaux^.linea.MaximoLinea;
          coloc:= lisaux^.linea.DevolverColocacion;
          lisaux^.linea.InsertarCaracter('');
          InsertarLinea;
          lisaux:=lisaux^.siguiente;
          lisaux^.linea.DefinirFormato (7,72);
          lisaux^.linea.DefinirFormato (coloc,4);

          actual:=lisaux;
     END
     ELSE
     BEGIN
          frase:='';
          frase:= actual^.linea.InsertarCaracter(car);
          WHILE frase <> '' DO
          BEGIN
               IF lisaux^.siguiente= NIL THEN
               BEGIN

                         alinea:=lisaux^.linea.MaximoLinea;
                         coloc:= lisaux^.linea.DevolverColocacion;
                         InsertarLinea;
                         lisaux:=lisaux^.siguiente;
                         lisaux^.linea.DefinirFormato (7,72);
                         lisaux^.linea.DefinirFormato (coloc,4);
                         actual:=lisaux;
                    IF frase<>'' THEN
                         frase:= lisaux^.linea.InsertarPalabraPrincipio (frase);
               END
               ELSE
               BEGIN
                    IF lisaux^.linea.ActualPosicion>=lisaux^.linea.LongitudActual+1 THEN
                         actual:=actual^.siguiente;
                    lisaux:=lisaux^.siguiente;
                    IF frase<>'' THEN
                       frase:= lisaux^.linea.InsertarPalabraPrincipio (frase);
               END;
          END;
     END;
END;


PROCEDURE TipoFichero.InsertarCaracterColocado(car:char);
(* Llama al procedimiento del TipoLinea para insertar un caracter *)
VAR
    frase,aux: string;
    lisaux, lisaux2: TipoPuntero;
    max, alin, bucle: byte;
    neg: TipoFormato;
BEGIN

     lisaux:=final;
     frase:='';
          frase:= lisaux^.linea.InsertarCaracter(car);
          WHILE frase <> '' DO
          BEGIN
               alin:=lisaux^.linea.DevolverColocacion;
               max:= lisaux^.linea.MaximoLinea;
               neg:=-1;
               IF lisaux^.linea.DevolverNegrita THEN
                  neg:=1;


               InsertarLinea;
               lisaux^.linea.ColocarLinea;
               lisaux:=final;
               lisaux^.linea.CrearLineaVacia(alindef, longdef);
               lisaux^.linea.DefinirFormato (7,max);
               lisaux^.linea.DefinirFormato (alin,0);
               lisaux^.linea.DefinirFormato (neg,0);
               IF frase <> ' ' THEN
                  frase:= lisaux^.linea.InsertarPalabraPrincipio (frase)
               ELSE
                   frase:='';
               final := lisaux;

               actual:=final;
          END;
END;

PROCEDURE TipoFichero.MoverActual(int :integer);
VAR
i:integer;

BEGIN

     FOR i:=1 TO int DO
          IF actual^.siguiente<> NIL THEN actual:= actual^.siguiente;
END;


PROCEDURE TipoFichero.VolcarFichero(var Fich:text);
VAR
line: string;
i:Integer;
aux:TipoPuntero;

BEGIN
     aux:=inicio;

     FOR i:=1 to cont do
     BEGIN
           line:= aux^.linea.DevolverLinea;
           IF i=cont THEN write (Fich,line)
                     ELSE Writeln (Fich,line);
           aux:= aux^.siguiente;
     END;
END;

PROCEDURE TipoFichero.VolcarFicheroResultado(var Fich:text);
VAR
line: string;
i:Integer;
aux:TipoPuntero;

BEGIN
     aux:=inicio;

     FOR i:=1 to cont do
     BEGIN
           line:= aux^.linea.DevolverLinea;
           IF line[length(line)] = '' THEN
              delete(line,length(line),1);
           Writeln (Fich,line);
           aux:= aux^.siguiente;
     END;

END;

FUNCTION TipoFichero.ActualCaracter: char;
BEGIN

     ActualCaracter:= actual^.linea.ActualCaracter;
END;

FUNCTION TipoFichero.ActualPosicion: byte;
BEGIN
     ActualPosicion:= actual^.linea.ActualPosicion;
END;

FUNCTION TipoFichero.MaximoLinea: byte;
BEGIN
     MaximoLinea:= actual^.linea.MaximoLinea;
END;

PROCEDURE TipoFichero.BorrarCaracter(tecla:char);
BEGIN
     actual^.linea.BorrarCaracter(tecla);
END;

PROCEDURE TipoFichero.InsertarFormato(formato:TipoFormato;aux:byte);
VAR
   lisaux: TipoPuntero;
   frase: string;
   alinea, coloc: byte;
BEGIN
     lisaux:=actual;

          frase:='';
          frase:= actual^.linea.InsertarFormato(formato,aux);
          WHILE frase <> '' DO
          BEGIN
               IF lisaux^.siguiente= NIL THEN
               BEGIN
                    alinea:=lisaux^.linea.MaximoLinea;
                    coloc:= lisaux^.linea.DevolverColocacion;
                    InsertarLinea;
                    lisaux:=lisaux^.siguiente;
                    lisaux^.linea.DefinirFormato (7,alinea);
                    lisaux^.linea.DefinirFormato (coloc,0);
                    IF lisaux^.linea.ActualPosicion>=lisaux^.linea.LongitudActual THEN
                       actual:=lisaux;
                    IF frase<>' ' THEN
                         frase:= lisaux^.linea.InsertarPalabraPrincipio (frase);
               END
               ELSE
               BEGIN
                    IF lisaux^.linea.ActualPosicion>=lisaux^.linea.LongitudActual THEN
                         actual:=actual^.siguiente;
                    lisaux:=lisaux^.siguiente;
                    IF frase<>' ' THEN
                       frase:= lisaux^.linea.InsertarPalabraPrincipio (frase);
               END;
          END;


END;

FUNCTION TipoFichero.NumeroLineas: integer;
BEGIN
     NumeroLineas:= cont;
END;



PROCEDURE  TipoFichero.VisualizarFichero(line:integer);
VAR
aux,i,bucle:integer;
lin:string;
lisaux: TipoPuntero;

BEGIN

     lisaux:=inicio;
     bucle:=1;
     IF (lisaux <> NIL) THEN
     BEGIN
         WHILE ((lisaux^.siguiente<>NIL) AND (bucle<line)) DO
         BEGIN
              lisaux:=lisaux^.siguiente;
              bucle:=bucle+1;
         END;
         bucle:=0;
         WHILE ((lisaux <> NIL) AND (bucle<19)) DO
         BEGIN
             lin:=lisaux^.linea.DevolverLinea;
             IF lin[length(lin)] = '' THEN
                delete(lin,length(lin),1);
             gotoxy (4,4+bucle);

             IF lisaux^.linea.DevolverNegrita THEN Textcolor(15)
             ELSE  Textcolor(7);

             write (lin);
             lisaux:=lisaux^.siguiente;
             bucle:=bucle+1;
         END;
     END;
END;

PROCEDURE TipoFichero.Derecha;
BEGIN
         actual^.linea.Derecha;
END;

PROCEDURE TipoFichero.Izquierda;
BEGIN
         actual^.linea.Izquierda;
END;

PROCEDURE TipoFichero.DefinirFormato (formato : TipoFormato; num:Byte);
BEGIN
        CASE formato OF
             -2: actual^.linea.DefinirFormato(formato,alindef);
             -3: actual^.linea.DefinirFormato(formato,alindef);
             -4: actual^.linea.DefinirFormato(formato,alindef);
             -5: actual^.linea.DefinirFormato(formato,alindef);
             -7: actual^.linea.DefinirFormato(formato,longdef);
             6: espacios:=num;
             -6: espacios:=espdef
        ELSE
            actual^.linea.DefinirFormato(formato,num);
        END;
END;

FUNCTION TipoFichero.LongitudActual:byte;
BEGIN
     LongitudActual:=actual^.linea.LongitudActual;
END;

FUNCTION TipoFichero.LineaActual: byte;
VAR
   lisaux: TipoPuntero;
   bucle: integer;
BEGIN
     lisaux:=inicio;
     bucle:=1;
     WHILE (lisaux<>actual) DO
     BEGIN
          lisaux:=lisaux^.siguiente;
          bucle:=bucle+1;
     END;
     LineaActual:=bucle;
END;

PROCEDURE TipoFichero.ColocarLinea;
BEGIN
     actual^.linea.ColocarLinea;
END;

FUNCTION TipoFichero.DevolverEspacios: byte;
BEGIN
     DevolverEspacios:=Espacios;
END;

PROCEDURE TipoFichero.DefinirFormatoDefecto ((*in*) alin, esp, long: byte);
BEGIN
     CASE alin OF
          1: alindef := 4;
          2: alindef := 5;
          3: alindef := 3;
          4: alindef := 2;
     END;
     espdef:=esp;
     espacios:=esp;
     longdef:=long;
     DefinirFormato(7,long);
     DefinirFormato(6,esp);
     DefinirFormato(alindef,0);
END;

END.
